#include "SonarExpert.h"
#include "math.h"

#define C1	57
#define C2	5

int initport(int fd) {
	struct termios options;
	/*// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 19200...
	cfsetispeed(&options, B19200);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);*/
	bzero(&options, sizeof(termios));
        cfmakeraw(&options);
        options.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
        options.c_iflag = IGNPAR | ICRNL;
        options.c_oflag = 0;
        options.c_lflag &= ICANON;

	// initialize all control characters
	options.c_cc[VINTR] = 0; // Ctrl-c
	options.c_cc[VQUIT] = 0; // Ctrl-backslash
	options.c_cc[VERASE] = 0; //del
	options.c_cc[VKILL] = 0; // @
	options.c_cc[VEOF] = 4; // Ctrl-d
	options.c_cc[VTIME] = 0; // inter-character timer unused
	options.c_cc[VMIN] = 0; //blocking read until 1 character arrives
	options.c_cc[VSWTC] = 0; // ’\0’
	options.c_cc[VSTART] = 0; // Ctrl-q
	options.c_cc[VSTOP] = 0; // Ctrl-s
	options.c_cc[VSUSP] = 0; //Ctrl-z
	options.c_cc[VEOL] = 0; // ’\0’
	options.c_cc[VREPRINT] = 0; // Ctrl-r
	options.c_cc[VDISCARD] = 0; // Ctrl-u
	options.c_cc[VWERASE] = 0; // Ctrl-w
	options.c_cc[VLNEXT] = 0; // Ctrl-v
	options.c_cc[VEOL2] = 0; // ’\0’

	tcflush(fd, TCIOFLUSH); // now clean the modem line...
	tcsetattr(fd, TCSANOW, &(options)); // and activate the settings for the port
	return 1;
}

int getbaud(int fd) {
	struct termios termAttr;
	int inputSpeed = -1;
	speed_t baudRate;
	tcgetattr(fd, &termAttr);
	/* Get the input speed.                              */
	baudRate = cfgetispeed(&termAttr);
	switch (baudRate) {
		case B0:      inputSpeed = 0; break;
		case B50:     inputSpeed = 50; break;
		case B110:    inputSpeed = 110; break;
		case B134:    inputSpeed = 134; break;
		case B150:    inputSpeed = 150; break;
		case B200:    inputSpeed = 200; break;
		case B300:    inputSpeed = 300; break;
		case B600:    inputSpeed = 600; break;
		case B1200:   inputSpeed = 1200; break;
		case B1800:   inputSpeed = 1800; break;
		case B2400:   inputSpeed = 2400; break;
		case B4800:   inputSpeed = 4800; break;
		case B9600:   inputSpeed = 9600; break;
		case B19200:  inputSpeed = 19200; break;
		case B38400:  inputSpeed = 38400; break;
	}
	return inputSpeed;
}

int writeport(int fd, char *chars) {
	int len = strlen(chars);
	chars[len] = 0x0d; // stick a <CR> after the command
	chars[len+1] = 0x00; // terminate the string properly
	int n = write(fd, chars, strlen(chars));
	if (n < 0) {
		fputs("write failed!\n", stderr);
		return 0;
	}
	return 1;
}

int readport(int fd, char *result) {
	result[0] = '\0';
	int iIn = read(fd, result, 254);
	if (iIn < 0) {
		if (errno == EAGAIN) {
			printf("SERIAL EAGAIN ERROR\n");
			return 0;
		} else {
			printf("SERIAL read error %d %s\n", errno, strerror(errno));
			return 0;
		}
	}
	else
	{
		result[iIn] = '\0';
	}
	return 1;
}

SonarExpert::SonarExpert(ModuleDispatch* dis,long period, WiimExpert* w) : StringModuleMember(dis, period, "SonarExpert")
{
	printf("\nSonarExpert built");
	fflush(stdout);
	
	lenbuffold = 0;
	we = w;
	h = 0;

	for (short count = 0; count < SENSORS_NUMBER; count ++)
	{
		dist[count] = MAXDISTANCE;
		//we->distances[count] = MAXDISTANCE;
	}
}

SonarExpert::~SonarExpert()
{
	printf("\nSonarExpert destroyed");
	fflush(stdout);
}

void SonarExpert::distancesVaried(int direction, int newDistance)
{
	printf("\nDistanze varied");
	fflush(stdout);

	dist[direction] = newDistance;

	NewMessage(MSG_TO_BRIAN, DISTANCES_DATA);
	mCurrentMessage<< "<D>"<<"NORTH "<<dist[NORTH]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"NORTHEAST "<<dist[NORTHEAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"EAST "<<dist[EAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"SOUTHEAST "<<dist[SOUTHEAST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"SOUTH "<<dist[SOUTH]<<" "<<0.9<<" <D>\n";
	mCurrentMessage<< "<D>"<<"SOUTHWEST "<<dist[SOUTHWEST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"WEST "<<dist[WEST]<<" "<<0.9<<" </D>\n";
	mCurrentMessage<< "<D>"<<"NORTHWEST "<<dist[NORTHWEST]<<" "<<0.9<<" </D>\n";
	CloseMessage();
	SendAllStringMessages();
}

void SonarExpert::RunInit()
{
	printf("\nSonarExpert initialized");
	fflush(stdout);
	
	sp = open("/dev/ttyUSB0",  O_RDWR | O_NOCTTY);
	if (sp == -1) {
		perror("open_port: Unable to open /dev/ttyUSB0 - ");
		return;
	}
	initport(sp);

	char r[] = "r";
	writeport(sp, r);
	
	AddMessageRequest(MSG_TO_SONAR, LOCAL);
}

void SonarExpert::RunDuty()
{	
	//printf("\nSonarExpert: RunDuty. Dist: ");

	if (readport(sp, buff) != 1)
	{
		//printf("\nRicevuto: %s\n", buff);
		//for (int i = 0; buff[i] != '\0'; printf("%d ", buff[i]), i++);
		//printf("\nANALIZZATO:\n");
		return;
	}
	int c = 0;
	for (; buff[c] != '\0'; c++)
	{
		if (buff[c] == '+')
		{
			for (int numVirgole = 0, numLettura = 0; buff[c] != '\n' && buff[c] != '\0'; c++)
			{
				if (numVirgole == 2)
				{
					int i = 0;
					for (; buff[c + i] != ',' && buff[c + i] != '\0' && buff[c + i] != '\n'; i++);
					if (buff[c + i] != '\0')
					{
						int k = 0;
						for (numLettura = 0; k < i; k++)
						{
							numLettura += (buff[c + k] - '0') * pow(10, (i - k - 1));
						}
					}
					c += i;
				}
				else if (numVirgole > 2)
				{
					int i = 0;
					for (; buff[c + i] != ',' && buff[c + i] != '\0' && buff[c + i] != '\n'; i++);
					if (buff[c + i] != '\0')
					{
						int k = 0;
						for (dist[(numLettura - 1) * 4 + numVirgole - 3] = 0; k < i; k++)
						{
							if (buff[c + k] != '0')
							{
								dist[(numLettura - 1) * 4 + numVirgole - 3] += (buff[c + k] - '0') * pow(10, (i - k - 1));
							}
						}
					}
					c += i;
				}
				
				if (buff[c] == ',')
				{	
					numVirgole ++;
				}
				if ((buff[c] == '\n') || (buff[c] == '\0'))
				{
					c--;
				}
			}
			if (buff[c] == '\0')
			{
				c--;
			}
		}
	}
	int i = c - 1;
	for (; buff[i] != '\n' && i > 0; i--);
	lenbuffold = c - i - 1;
	int cont = 0;
	for (i++; i < c; i++, cont++)
	{
		buff[cont] = buff[i];
	}
	
	int numAttivi = 0;
	if (1)
	{
	h = 0;
	printf("1: ");
	for (c = 0; c < SENSORS_NUMBER * 2; c++)
	{
		if (dist[c] == MAXDISTANCE)
		{
			numAttivi++;
			//we->distances[c] = dist[c];
			//dist[c] = MAXDISTANCE;
		}
		if (c % 2 == 0)
		{
			printf("\t%d ", dist[c]);
		}
	}
	printf("\n");
	if (numAttivi == 8)
	{
		printf("\nProbabile errore nei sonar!!!");
	}
	}
	else h++;

	NewMessage(MSG_TO_WIIM, DISTANCES_DATA);
	mCurrentMessage<<" North "<<dist[NORTH];
	mCurrentMessage<<" NE "<<dist[NORTHEAST];
	mCurrentMessage<<" East "<<dist[EAST];
	mCurrentMessage<<" SE "<<dist[SOUTHEAST];
	mCurrentMessage<<" South "<<dist[SOUTH];
	mCurrentMessage<<" SW "<<dist[SOUTHWEST];
	mCurrentMessage<<" West "<<dist[WEST];
	mCurrentMessage<<" NW "<<dist[NORTHWEST]<<"\n";
	CloseMessage();
	SendAllStringMessages();
	
	unsigned char* buffer;
	if(ReceiveLastMessage(MSG_TO_SONAR, buffer,false) > 0)
	{
		float n;
		int i, tot = 0;
		for(i=0;buffer[i]!=NULL;i++);
    		char* buf=(char*)malloc(i+1);
    		strcpy(buf,(char*)buffer);	
		if(strtovalue(buf,"Charge ",&n))
		{
			tot = (int) n;
		}
		
		if(strtovalue(buf,"Point ",&n))
		{
			tot |= (int) n;
		}
		
		char c = (char) tot;
		write(sp, &c, 1);
	}
}

int SonarExpert::getDistance(int num)
{
	return dist[num];
}

void SonarExpert::RunClose()
{
	printf("\nSonarExpert closed");
	fflush(stdout);
}